@@ -1,14 +1,70 @@
 "qtscore" <-
-function(formula,data,snpsubset,idsubset,strata,trait.type,times=1,quiet=FALSE,bcast=10,clambda=TRUE) {
+function(formula,data,snpsubset,idsubset,strata,trait.type="gaussian",times=1,quiet=FALSE,bcast=10,clambda=TRUE,propPs=1.0) {
   	if (class(data)!="gwaa.data") {
 		stop("wrong data class: should be gwaa.data")
   	}
 	if (!missing(snpsubset)) data <- data[,snpsubset]
 	if (!missing(idsubset)) data <- data[idsubset,]
 	if (missing(strata)) {nstra=1; strata <- rep(0,data@gtdata@nids)}
+	ttargs <- c("gaussian","binomial")
+	if (!(match(trait.type,ttargs,nomatch=0)>0)) {
+		out <- paste("trait.type argument should be one of",ttargs,"\n")
+		stop(out)
+	}
+	if (trait.type=="gaussian") fam <- gaussian()
+	if (trait.type=="binomial") fam <- binomial()
 
-	avrs <- all.vars(as.formula(formula))
-	if (!any(avrs=="CRSNP")) stop("formula must contain CRSNP variable to be replaced with the analysis SNPs")
+	if (!missing(data)) attach(data@phdata,pos=2,warn.conflicts=FALSE)
+	if (class(formula) == "formula") {
+		mf <- model.frame(formula,data,na.action=na.omit,drop.unused.levels=TRUE)
+		y <- model.response(mf)
+#		sdy <- sd(y)
+#		meany <- mean(y)
+#		if (trait.type=="gaussian") y <- (y-mean(y))/sd(y)
+		desmat <- model.matrix(formula,mf)
+		lmf <- glm.fit(desmat,y,family=fam)
+#		iniest <- lmf$coeff
+		mids <- rownames(data@phdata) %in% rownames(mf)
+		if (trait.type=="binomial") {
+			bin <- 1
+		} else {
+			bin <- 0
+		}
+		resid <- lmf$resid
+	} else {
+		y <- formula
+		mids <- (!is.na(y))
+		y <- y[mids]
+#		sdy <- sd(y)
+#		meany <- mean(y)
+#		if (trait.type=="gaussian") y <- (y-mean(y))/sd(y)
+#		desmat <- matrix(1,nrow=length(y))
+#		if (trait.type=="binomial") {
+#			tvar <- var(y)
+#			tmp <- mean(y)
+#			iniest <- c(log(tmp/(1.-tmp)))
+#		} else {
+#			tvar <- var(y)
+#			iniest <- c(mean(y))
+#		}
+		resid <- y
+		if (length(unique(resid))==1) stop("trait is monomorphic")
+		if (length(unique(resid))==2) bin <- 1 else bin <- 0
+		if (!missing(trait.type)) {
+			if (trait.type=="binomial") {
+				if (bin == 0) stop("more then two levels in trait")
+				bin <- 1
+			} else if (trait.type=="gaussian") {
+				if (bin == 1) warning("binary traits is analysed as gaussian")
+				bin <- 0
+			} else {
+				stop("trait type should be either binomial or gaussian")
+			}
+		}
+		if (bin == 1) trait.type="binomial" else if (bin == 0) trait.type="gaussian"
+
+	}
+	if (!missing(data)) detach(data@phdata)
 	if (length(strata)!=data@gtdata@nids) stop("Strata variable and the data do not match in length")
 	if (any(is.na(strata))) stop("Strata variable contains NAs")
 	if (any(strata!=0)) {
@@ -21,45 +77,13 @@ function(formula,data,snpsubset,idsubset,strata,trait.type,times=1,quiet=FALSE,b
 	}
 	nstra <- length(levels(as.factor(strata)))
 
-	attach(data@phdata,pos=2,warn.conflicts=FALSE)
-	tmeas <- !is.na(get(avrs[1],pos=2))
-	avrs <- avrs[avrs!="CRSNP"]
-	if (length(avrs)>=2) for (i in 2:length(avrs)) tmeas <- tmeas*(!is.na(get(avrs[i],pos=2)))
-	tmeas <- as.logical(tmeas)
-	resid <- get(avrs[1],pos=2)[tmeas]
+	tmeas <- as.logical(mids)
 	strata <- strata[tmeas]
-	if (length(avrs)>=1) for (i in 1:length(avrs)) assign(avrs[i],get(avrs[i],pos=2)[tmeas])
-	detach(data@phdata)
-
-	if (length(unique(resid))==1) stop("trait is monomorphic")
-	if (length(unique(resid))==2) bin <- 1 else bin <- 0
-	if (!missing(trait.type)) {
-		if (trait.type=="binomial") {
-			if (bin == 0) stop("more then two levels in trait")
-			bin <- 1
-		} else if (trait.type=="gaussian") {
-			if (bin == 1) warning("binary traits is analysed as gaussian")
-			bin <- 0
-		} else {
-			stop("trait type should be either binomial or gaussian")
-		}
-	}
-	if (bin == 1) trait.type="binomial" else if (bin == 0) trait.type="gaussian"
 
 	if (any(tmeas == FALSE)) {
 		if (!quiet) warning(paste(sum(!tmeas),"people (out of",length(tmeas),") excluded because they have trait or covariate missing\n"),immediate. = TRUE)
 		data <- data[tmeas,]
 	}
-	if (length(avrs) > 1) {
-		if (bin) {
-			CRSNP <- rep(0,data@gtdata@nids)
-			resid <- glm(as.formula(formula),data=data@phdata,family=binomial())$residuals
-			resid <- exp(resid)/(1+exp(resid))
-		} else {
-			CRSNP <- rep(0,data@gtdata@nids)
-			resid <- lm(as.formula(formula),data=data@phdata)$residuals
-		}
-	} 
 
 	lenn <- data@gtdata@nsnps;
 	for (j in c(1:(times+1*(times>1)))) {
@@ -67,7 +91,8 @@ function(formula,data,snpsubset,idsubset,strata,trait.type,times=1,quiet=FALSE,b
 		chi2 <- .C("qtscore",as.raw(data@gtdata@gtps),as.double(resid),as.integer(bin),as.integer(data@gtdata@nids),as.integer(data@gtdata@nsnps), as.integer(nstra), as.integer(strata), chi2 = double(6*data@gtdata@nsnps), PACKAGE="GenABEL")$chi2
 		if (any(data@gtdata@chromosome=="X")) {
 		  ogX <- data@gtdata[,data@gtdata@chromosome=="X"]
-		  chi2X <- .C("qtscore",as.raw(ogX@gtps),as.double(resid),as.integer(bin),as.integer(ogX@nids),as.integer(ogX@nsnps), as.integer(2), as.integer(ogX@male), chi2 = double(6*ogX@nsnps), PACKAGE="GenABEL")$chi2
+		  sxstra <- strata; sxstra[ogX@male==1] <- strata[ogX@male==1]+nstra
+		  chi2X <- .C("qtscore",as.raw(ogX@gtps),as.double(resid),as.integer(bin),as.integer(ogX@nids),as.integer(ogX@nsnps), as.integer(nstra*2), as.integer(sxstra), chi2 = double(6*ogX@nsnps), PACKAGE="GenABEL")$chi2
 		  revec <- (data@gtdata@chromosome=="X")
 		  revec <- rep(revec,6)
 		  chi2 <- replace(chi2,revec,chi2X)
@@ -83,7 +108,7 @@ function(formula,data,snpsubset,idsubset,strata,trait.type,times=1,quiet=FALSE,b
 				lambda$se <- NA
 				chi2.c1df <- chi2.1df;
 			} else {
-				lambda <- estlambda(chi2.1df,plot=FALSE,prop=0.9)
+				lambda <- estlambda(chi2.1df,plot=FALSE,prop=propPs)
 				def <- 1/lambda$estimate
 				if (def > 1 && clambda) {
 					chi2.c1df <- chi2.1df;
